#include "global.h"
#include "rprintf.h"	// include printf function library
#include "./ata.h"
#include "./misc.h"

typedef struct 
{
  unsigned long sizeinsectors;
  unsigned char LBAsupport;

  char model[41];
} typeDriveInfo;

typeDriveInfo ataDriveInfo;

void ataInit() {
  int i;
  outb(ATA_ADDR_PORT, ADDR_MASK);
  outb(DDRB,inb(DDRB)|ADDR_MASK); // address on port B
  outb(ATA_RW_PORT, inb(PORTD) | 1<<ATA_RD|1<<ATA_WR);
  outb(DDRD, inb(DDRD)|1<<ATA_RD|1<<ATA_WR); // read and write commands
  
  outb(PORTA, 0xFF);
  outb(DDRA, 0xFF);
  for (i=0; i<255; i++) {delay(255);}
  timerPause(100);

  // set 8 bit mode
  while(ataReadByte(STATUS_REG) & 0x80) {}
  ataWriteByte(FEATURE_REG, 0x01);
  while(ataReadByte(STATUS_REG) & 0x80) {}
  ataWriteByte(COMMAND_REG, 0xEF);
  // set LBA mode
  while(ataReadByte(STATUS_REG) & 0x80) {}
  ataWriteByte(LBA3_REG, 0xE0);
  // always one sector at a time
  while(ataReadByte(STATUS_REG) & 0x80) {}
  ataWriteByte(SECTOR_CNT_REG, 0x01);
}

u08 ataReadByte(u08 reg)
{
  register u08 ret;
  outb(ATA_RW_PORT, 1<<ATA_RD|1<<ATA_WR);
  outb(DDRA, 0x00); //PORT A INPUT
  outb(ATA_ADDR_PORT, reg & ADDR_MASK); // set address
  
  cbi(ATA_RW_PORT, ATA_RD);  // assert RD
  __asm__ __volatile__ ("nop"); 
  ret = inb(PINA);
  sbi(ATA_RW_PORT, ATA_RD);  // release RD
  
  outb(DDRA, 0xFF); //PORT A OUTPUT
  outb(ATA_ADDR_PORT, ATA_RELEASE); // release address
  return ret;
}

void ataInitReadFast()
{
  outb(ATA_RW_PORT, 1<<ATA_RD|1<<ATA_WR);
  outb(DDRA, 0x00); //PORT A INPUT
  outb(ATA_ADDR_PORT, DATA_REG); // set address
}

u08 ataReadFast()
{
  register u08 ret;
  cbi(ATA_RW_PORT, ATA_RD);  // assert RD
  __asm__ __volatile__ ("nop"); 
  ret = inb(PINA);
  sbi(ATA_RW_PORT, ATA_RD);  // release RD
  return ret;
}

void ataEndReadFast()
{
  outb(ATA_DATA_PORT, 0xFF);
  outb(DDRA, 0xFF); //PORT A OUTPUT
  outb(ATA_ADDR_PORT, ATA_RELEASE); // release address
}

void ataWriteByte(u08 reg, u08 data)
{
  outb(ATA_RW_PORT, 1<<ATA_RD|1<<ATA_WR);
  outb(DDRA, 0xFF); //PORT A OUTPUT
  outb(ATA_DATA_PORT, data);
  outb(ATA_ADDR_PORT, reg & ADDR_MASK); // set address
  cbi(ATA_RW_PORT, ATA_WR);  // assert WR
  __asm__ __volatile__ ("nop"); 
  sbi(ATA_RW_PORT, ATA_WR);  // release WR
  
  outb(ATA_DATA_PORT, 0xFF);
  outb(DDRA, 0xFF); //PORT A OUTPUT
  outb(ATA_ADDR_PORT, ATA_RELEASE); // release address
}

void ataIdentify(unsigned char* buffer) {
  
  u08 i;
    
  // read drive identity
  rprintf("\r\nScanning IDE interface...\r\n");
  // Wait for drive to be ready
  while(ataReadByte(STATUS_REG) & 0x80) {}
  
  // issue identify command
  ataWriteByte(COMMAND_REG, 0xEC);
  // wait for drive to request data transfer
  while(ataReadByte(STATUS_REG) & 0x80) {}
  while(!ataReadByte(STATUS_REG) & 0x08) {}
  
  //timerPause(200);
  // read in the data
  ataReadDataBuffer(buffer, 512);

  // set local drive info parameters
  ataDriveInfo.LBAsupport = *( ((unsigned int*) buffer) + ATA_IDENT_FIELDVALID );
  ataDriveInfo.sizeinsectors =  *( (unsigned long*) (buffer + ATA_IDENT_LBASECTORS*2) );
  // copy model string
  for(i=0; i<40; i+=2)
    {
      // correct for byte order
      ataDriveInfo.model[i  ] = buffer[(ATA_IDENT_MODEL*2) + i + 1];
      ataDriveInfo.model[i+1] = buffer[(ATA_IDENT_MODEL*2) + i    ];
    }
  // terminate string
  ataDriveInfo.model[40] = 0;
  
  // process and print info
  if(ataDriveInfo.LBAsupport)
    {
      // LBA support
      rprintf("Drive 0: %dMB ", ataDriveInfo.sizeinsectors/(1000000/512) );
      rprintf("LBA mode -- MODEL: ");
    }
  else
    {
      // CHS, no LBA support
      // ERROR!
      rprintf("NO LBA SUPPORT!", ataDriveInfo.sizeinsectors/(1000000/512) );
    }
  // print model information      
  printString(ataDriveInfo.model);
  rprintf("\r\n");
}

//void ataReadSector(u08 *buffer, 

void ataReadDataBuffer(u08 *buffer, u16 numBytes)
{
  unsigned int i;

  ataInitReadFast();  
  // read data from drive
  for (i=0; i<(numBytes/16); i++)
    {
      // optimize by reading 16 bytes in-line before looping
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
      *buffer++ = ataReadFast();
    }
  ataEndReadFast();  
  
}


void ataPrintSector(u08 *Buffer)
{
        u08 i;
        u16 j;
        u08 *buf;
        u08 s;

        buf = Buffer;
        
        // print the low order address indicies
        rprintf("\r\n     00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F  0123456789ABCDEF\r\n");
        rprintf("     -----------------------------------------------  ---- ASCII -----\r\n");
        
        // print the data
        for(j=0; j<0x20; j++)
        {
	  // print the high order address index for this line
	  rprintf("%x   ", j);

	  // print the hex data
	  for(i=0; i<0x10; i++)
	    {
	      rprintf("%x ", (buf[(j<<4)+i]));
	    }
	  
	  // leave some space
	  rprintf(" ");
	  
	  // print the ascii data
	  for(i=0; i<0x10; i++)
	    {
	      s = buf[(j<<4)+i]; 
	      // make sure character is printable
	      if(s >= 0x20 && s <= 0x7F)
		{
		  rprintf("%c", s);
		}
	      else
		{
		  rprintf(" ");
		}
	      
	    }
	  rprintf("\r\n");
        }
}




void readRegisters() {
  rprintf("\r\nDATA REGISTER CONTENTS\r\n");
  rprintf("0: %x\r\n", ataReadByte(0x10));
  rprintf("1: %x\r\n", ataReadByte(0x11));
  rprintf("2: %x\r\n", ataReadByte(0x12));
  rprintf("3: %x\r\n", ataReadByte(0x13));
  rprintf("4: %x\r\n", ataReadByte(0x14));
  rprintf("5: %x\r\n", ataReadByte(0x15));
  rprintf("6: %x\r\n", ataReadByte(0x16));
  rprintf("7: %x\r\n", ataReadByte(0x17));
}

unsigned char ataReadSector(
			     unsigned long lba,
			     unsigned char *Buffer)
{
  unsigned char temp;
  unsigned char lbabyte;

  while(ataReadByte(STATUS_REG) & 0x80) {}

  rprintf("Getting sector: ");
  rprintfNum(10, 10, FALSE, ' ', lba);
  rprintf("\r\n");
  


  lbabyte = (int) ( lba & 0x000000ffL );
  lba = lba >> 8;
  ataWriteByte(LBA0_REG, lbabyte);
  rprintf("%d ", lbabyte);
  lbabyte = (int) ( lba & 0x000000ffL );
  lba = lba >> 8;
  ataWriteByte(LBA1_REG, lbabyte);
  rprintf("%d ", lbabyte);
  lbabyte = (int) ( lba & 0x000000ffL );
  lba = lba >> 8;
  ataWriteByte(LBA2_REG, lbabyte);
  rprintf("%d ", lbabyte);
  lbabyte = (int) ( lba & 0x000000ffL );
  lba = lba >> 8;
  ataWriteByte(LBA3_REG, lbabyte | 0xE0);
  rprintf("%d \r\n", lbabyte);

  rprintf("A");
  ataWriteByte(COMMAND_REG, 0x21);
  rprintf("B");
   // wait for drive to request data transfer
  while(ataReadByte(STATUS_REG) & 0x80) {}
  rprintf("C");
  while(!ataReadByte(STATUS_REG) & 0x08) {}
  rprintf("D");
 
  ataReadDataBuffer(Buffer, 512);
  rprintf(" DONE\r\n");
  
  return 0;

}
