#ifndef ATA_H
#define ATA_H

#include <io.h>		// include I/O definitions (port names, pin names, etc)
#include <sig-avr.h>	// include "signal" names (interrupt names)
#include <progmem.h>

#include "timer.h"
#include "global.h"

#define ATA_RD 6
#define ATA_WR 4

#define ADDR_MASK 0x1F
#define ATA_ADDR_PORT PORTB
#define ATA_DATA_PORT PORTA
#define ATA_RW_PORT PORTD

#define ATA_RELEASE 0x18

#define DATA_REG  0x10
#define ERROR_REG 0x11
#define FEATURE_REG 0x11
#define SECTOR_CNT_REG 0x12
#define LBA0_REG 0x13
#define LBA1_REG 0x14
#define LBA2_REG 0x15
#define LBA3_REG 0x16
#define STATUS_REG 0x17
#define COMMAND_REG 0x17

void ataInit(void);
u08 ataReadByte(u08 reg);
void ataWriteByte(u08 reg, u08 data);
// ATA Identity fields
// all offsets refer to word offset (2 byte increments)
#define ATA_IDENT_DEVICETYPE    0               // specifies ATA/ATAPI, removable/non-removable
#define ATA_IDENT_CYLINDERS             1               // number of logical cylinders
#define ATA_IDENT_HEADS                 3               // number of logical heads
#define ATA_IDENT_SECTORS               6               // number of sectors per track
#define ATA_IDENT_SERIAL                10              // drive model name (20 characters)
#define ATA_IDENT_MODEL                 27              // drive model name (40 characters)
#define ATA_IDENT_FIELDVALID    53              // indicates field validity of higher words (bit0: words54-58, bit1: words 64-70)
#define ATA_IDENT_LBASECTORS    60              // number of sectors in LBA translation mode
#endif
