//*****************************************************************************
//
// This is a subset of Pascal's avrlib i2c library, found at 
// http://rsl.engr.scu.edu/~pstang/avr/avrlib
//
// This code is distributed under the GNU Public License
//              which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************
#include <io.h>
#include <sig-avr.h>
#include <interrupt.h>
#include <twi.h>

#include "i2c.h"

void i2cInit(void)
{
  outb(TWBR, 10); // 200khz
  // enable TWI (two-wire interface)
  sbi(TWCR, TWEN);
  // enable slave address ACK
  sbi(TWCR, TWEA);
}

// MASTER TRANSMITTER
void i2cSend(u08 deviceAddr, u08 length, u08 *data) {
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (deviceAddr&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data
  while(length)
    {
      outb(TWDR, *data++);
      outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
      while( !(inb(TWCR) & BV(TWINT)) );
      length--;
    }
  
  sbi(TWCR, TWSTO); // send stop
}

void i2cReceive(u08 deviceAddr, u08 length, u08* data) {

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );

  // send device address with read
  outb(TWDR, (deviceAddr&0xFE) | 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // accept receive data and ack it
  while(length > 1)
    {
      outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
      while( !(inb(TWCR) & BV(TWINT)) );
      *data++ = inb(TWDR);
      length--;
    }
  
  // accept receive data and nack it (last-byte signal)
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  *data++ = inb(TWDR);
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
}

