#include "global.h"
#include "./lcd.h"

// Function to initialize the LCD.

// preconditions: none
// postconditions: PORT D set to output on RW, RS, EN
// RW=RS=1, EN=0
// PORT A set to output FF
void lcdInit(void) {
  // set up PWM for contrast control
  timer1SetPrescaler(TIMER_CLK_DIV8);
  timer1PWMInit(9);
  cbi(TCCR1A, COM1B1);	// Pascal's code initializes PWM B, so I need
  cbi(TCCR1A, COM1B0);	// to manually disable it.  Thanks Pascal! ;-)
  

  outb(PORTD, inb(PORTD) | 1<<LCD_RS | 1<<LCD_RW);
  cbi(PORTD, 1<<LCD_E);
  outb(DDRD, inb(DDRD) 
       | 1<<LCD_RS | 1<<LCD_RW 
       | 1<<LCD_E | 1<< LCD_BIAS); // set port d as control

  outb(PORTA, 0xFF);
  outb(DDRA, 0xFF);
  
  lcdCommandWrite(0x38); 
  lcdCommandWrite(0x06); 
  lcdCommandWrite(0x0C); 
  lcdCommandWrite(0x01); 
}

// Function to write a command byte to the LCD
// postconditions: PORTA set to output 0xFF
void lcdCommandWrite(u08 data) {
  lcdCheckBusy();
  cbi(LCD_COMMAND_PORT, LCD_RS); 	
  cbi(LCD_COMMAND_PORT, LCD_RW);
  outb(LCD_DATA_PORT, data);
  sbi(LCD_COMMAND_PORT, LCD_E);
  __asm__ __volatile__ ("nop"); 
  cbi(LCD_COMMAND_PORT, LCD_E);
  sbi(LCD_COMMAND_PORT, LCD_RS); 	
  sbi(LCD_COMMAND_PORT, LCD_RW);
  outb(LCD_DATA_PORT, 0xFF);
}

// Function to write a data byte to the LCD
void lcdDataWrite(u08 data) {
  lcdCheckBusy();
  sbi(LCD_COMMAND_PORT, LCD_RS); 	
  cbi(LCD_COMMAND_PORT, LCD_RW);
  outb(LCD_DATA_PORT, data);
  sbi(LCD_COMMAND_PORT, LCD_E);
  __asm__ __volatile__ ("nop"); 
  cbi(LCD_COMMAND_PORT, LCD_E);
  sbi(LCD_COMMAND_PORT, LCD_RS); 	
  sbi(LCD_COMMAND_PORT, LCD_RW);
  outb(LCD_DATA_PORT, 0xFF);
}

void lcdReset(){
  lcdCommandWrite(LCD_RESET);
}

void lcdReturn(){
  lcdCommandWrite(LCD_RETURN);
}

void lcdSetAddress(u08 addr){
  lcdCommandWrite(addr | 0x80);
}

// Function to loop while LCD is busy
void lcdCheckBusy(void) {
  u08 busy;
  outb(DDRA, 0x00);
  do 
    {
      cbi(LCD_COMMAND_PORT, LCD_RS);
      sbi(LCD_COMMAND_PORT, LCD_RW);
      sbi(LCD_COMMAND_PORT, LCD_E);
      // lcdDelay();
      __asm__ __volatile__ ("nop"); 
      __asm__ __volatile__ ("nop"); 
      __asm__ __volatile__ ("nop"); 
      __asm__ __volatile__ ("nop"); 
      __asm__ __volatile__ ("nop"); 
      busy = inb(PINA) & (1<<LCD_BUSY);
      cbi(LCD_COMMAND_PORT, LCD_E);
      sbi(LCD_COMMAND_PORT, LCD_RS);
      sbi(LCD_COMMAND_PORT, LCD_RW);
    }	
  while (busy);
  outb(DDRA, 0xFF);
}

void lcdSetContrast(u08 contrast) {
  timer1PWMASet(contrast);
}


// Function to load the custom characters into CGRAM
void lcdCustomChars(u08 *lcd_char_map) {
  int i;
  lcdCommandWrite(0x00|(1<<LCD_SET_C_ADDRESS));
  for (i = 0; i<64; i++) 
    {
      lcdDataWrite(PRG_RDB(&lcd_char_map[i]));
    }
  lcdCommandWrite(0x01);	
}

void lcdDelay() {
    __asm__ __volatile__ ("nop"); 
    __asm__ __volatile__ ("nop"); 
    __asm__ __volatile__ ("nop"); 
    __asm__ __volatile__ ("nop"); 
    __asm__ __volatile__ ("nop"); 
}

void lcdStablePrint(char* s, u08 offset) {
  u08 i = 0;
  lcdSetAddress(offset);
  
  while (s[i] != 0 && i <= 40) {
    lcdDataWrite(s[i]);
    i++;
    if (i+offset == 40) 
      lcdSetAddress(0);
  }
}
