#include <io.h>
#include <sig-avr.h>
#include <interrupt.h>
#include <twi.h>

#include "mas3587f.h"
#include "rprintf.h"	// include printf function library




void initMAS() {
  u32 data;
  // set analog supply range, enable audio codec and core
  data = 0x8C00;
  MASwritereg(CONTROL_REG, &data);
  timerPause(200);
  // 16 MHZ clock
  data = 0x00003E80;
  MASwriteD0(MAS_OFREQ_CONTROL, &data);
  
  rprintf("Version: ");
  MASreadVersion(&data);
  rprintfu32(data);
  rprintf("\r\n");


}






void MASreadreg(u08 subaddr, u16* data) {
  u08* tmpdata = (u08*) data;

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send subaddress
  outb(TWDR, subaddr);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );

  // send device address with read
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) | 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // accept receive data and ack it
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmpdata[1] = inb(TWDR);

  // accept receive data and nack it (last-byte signal)
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmpdata[0] = inb(TWDR);
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
}
  
void MASwritereg(u08 subaddr, 
		 u16 *data) {
  u08* tmpdata = (u08*) data;

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send subaddress
  outb(TWDR, subaddr);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data
  outb(TWDR, tmpdata[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send data
  outb(TWDR, tmpdata[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
}


void MASreadD0(u16 subaddr, u32* data) {
  u08 tmp1;
  u08 tmp2;
  u08 tmp3;


  u08* tmpsubaddr = (u08*)&subaddr;
  u08* tmpdata = (u08*)data;
  
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_write
  outb(TWDR, MAS_DATA_WRITE);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send 0xC0
  outb(TWDR, 0xC0);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send 0x00
  outb(TWDR, 0x00);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send 0x00
  outb(TWDR, 0x00);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send 0x01
  outb(TWDR, 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send high
  outb(TWDR, tmpsubaddr[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send low
  outb(TWDR, tmpsubaddr[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // transmit stop condition
  sbi(TWCR, TWSTO);



  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_read
  outb(TWDR, MAS_DATA_READ);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send device address with read
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) | 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // accept receive data and ack it
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  // ignore first one
  
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp1 = inb(TWDR);
  tmpdata[2] = inb(TWDR);

  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp2 = inb(TWDR);
  tmpdata[1] = inb(TWDR);  
  
  // last byte NAK
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp3 = inb(TWDR);
  tmpdata[0] = inb(TWDR);  
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
  
  tmpdata[3] = 0x00;

  //rprintf("DATA %x %x %x => ", tmp1, tmp2,tmp3);
}

void MASwriteD0(u16 subaddr, u32 *data) {
  u08* tmpsubaddr = (u08*)&subaddr;
  u08* tmpdata = (u08*)data;

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_write
  outb(TWDR, MAS_DATA_WRITE);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send 0xE0
  outb(TWDR, 0xE0);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send 0x00
  outb(TWDR, 0x00);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send 0x00
  outb(TWDR, 0x00);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send 0x01
  outb(TWDR, 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send high
  outb(TWDR, tmpsubaddr[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send low
  outb(TWDR, tmpsubaddr[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send first one
  outb(TWDR, 0x00);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send second one
  outb(TWDR, tmpdata[2]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send third one
  outb(TWDR, tmpdata[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send second one
  outb(TWDR, tmpdata[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
  
}

void MASreadCODEC(u16 subaddr, u16* data) {
  u08 tmp1;
  u08 tmp2;

  u08* tmpsubaddr = (u08*)&subaddr;
  u08* tmpdata = (u08*)data;
  
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );

  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_write
  outb(TWDR, MAS_CODEC_WRITE);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send high
  outb(TWDR, tmpsubaddr[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send low
  outb(TWDR, tmpsubaddr[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // transmit stop condition
  sbi(TWCR, TWSTO);


  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_read
  outb(TWDR, MAS_CODEC_READ);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send device address with read
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) | 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp1 = inb(TWDR);
  tmpdata[1] = inb(TWDR);  
  
  // last byte NAK
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp2 = inb(TWDR);
  tmpdata[0] = inb(TWDR);  
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
  
  rprintf("DATA %x %x => ", tmp1, tmp2);
}

void MASwriteCODEC(u16 subaddr, u16 *data) {
  u08* tmpsubaddr = (u08*)&subaddr;
  u08* tmpdata = (u08*)data;

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_write
  outb(TWDR, MAS_CODEC_WRITE);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send high
  outb(TWDR, tmpsubaddr[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send low
  outb(TWDR, tmpsubaddr[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send first one
  outb(TWDR, tmpdata[1]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send second one
  outb(TWDR, tmpdata[0]);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // transmit stop condition
  sbi(TWCR, TWSTO);
  
}

u32 MASreadVersion(u32* data) {

  u08* tmpdata = (u08*)data;

  u08 tmp1;
  u08 tmp2;
  u08 tmp3;
  u08 tmp4;
  u32 rtmp1;
  u32 rtmp2;
  u32 rtmp3;

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_write
  outb(TWDR, MAS_DATA_WRITE);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // send 0x70
  outb(TWDR, 0x70);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  // send 0x00
  outb(TWDR, 0x00);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );

  // transmit stop condition
  sbi(TWCR, TWSTO);

  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  // send device address with write
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) );
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send data_read
  outb(TWDR, MAS_DATA_READ);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  sbi(TWCR, TWSTA); // send start
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // send device address with read
  outb(TWDR, (MAS_SLAVE_ADDR&0xFE) | 0x01);
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  
  // accept receive data and ack it
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp1 = inb(TWDR);
  tmpdata[3] = inb(TWDR);
  
    outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp2 = inb(TWDR);
  tmpdata[2] = inb(TWDR);

  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp3 = inb(TWDR);
  tmpdata[1] = inb(TWDR);
  
  // last byte NAK
  outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
  while( !(inb(TWCR) & BV(TWINT)) );
  tmp4 = inb(TWDR);
  tmpdata[0] = inb(TWDR);
  
  // transmit stop condition
  sbi(TWCR, TWSTO);

  //rprintf("DATA %x %x %x %x => ", tmp1, tmp2,tmp3,tmp4);
  
}

