#ifndef MAS3587F_H
#define MAS3587F_H

#include <io.h>		// include I/O definitions (port names, pin names, etc)
#include <sig-avr.h>	// include "signal" names (interrupt names)

#include "i2c.h"

#define MAS_SLAVE_ADDR 0x3E

// registers
#define CONTROL_REG 0x6a
#define DCCF_REG 0x76
#define DCFR_REG 0x77

//
#define MAS_DATA_WRITE             0x68
#define MAS_DATA_READ              0x69
#define MAS_CODEC_WRITE            0x6C
#define MAS_CODEC_READ             0x6D

// commands
#define MAS_RUN                    0x00
#define MAS_READ_ANCILLARY         0x50
#define MAS_FAST_PROGRAM_DOWNLOAD  0x60
#define MAS_READ_VERSION           0x70
#define MAS_READ_REGISTER          0xA0
#define MAS_WRITE_REGISTER         0xB0
#define MAS_READ_D0                0xC0
#define MAS_READ_D1                0xD0
#define MAS_WRITE_D0               0xE0
#define MAS_WRITE_D1               0xF0


// D0 control memory addresses
#define MAS_APP_SELECT        0x07F6
#define MAS_APP_RUNNING       0x07F7
#define MAS_ENCODER_CONTROL   0x07F0
#define MAS_IO_MAIN_CONTROL   0x07F1
#define MAS_INTERFACE_CONTROL 0x07F2
#define MAS_OFREQ_CONTROL     0x07F3
#define MAS_OUTPUT_CLK_CONFIG 0x07F4
#define MAS_SPD_OUT_BITS      0x07F8
#define MAS_SOFT_MUTE         0x07F9
#define MAS_OUT_LL            0x07FC
#define MAS_OUT_LR            0x07FD
#define MAS_OUT_RL            0x07FE
#define MAS_OUT_RR            0x07FF

// D0 status memory addresses
#define MAS_MPEG_FRAMECOUNT   0x0FD0
#define MAS_MPEG_STATUS1      0x0FD1
#define MAS_MPEG_STATUS2      0x0FD2
#define MAS_CRC_ERRORCOUNT    0x0FD3
#define MAS_NUM_ANCILLARY     0x0FD4
#define MAS_ANCILLARY_DATA    0x0FD5

// Codec Registers
#define MAS_CONV_CONF         0x0000
#define MAS_ADC_IN_MODE       0x0008
#define MAS_DAC_IN_ADC        0x0006
#define MAS_DAC_IN_DSP        0x0007
#define MAS_DAC_OUT_MODE      0x000E
#define MAS_BASS              0x0014
#define MAS_TREBLE            0x0015
#define MAS_LOUDNESS          0x001E
#define MAS_MDB_STR           0x0022
#define MAS_MDB_HAR           0x0023
#define MAS_MDB_FC            0x0024
#define MAS_MDB_SHAPE_SWITCH  0x0021
#define MAS_VOLUME            0x0010
#define MAS_BALANCE           0x0011
#define MAS_AVC               0x0012
#define MAS_QPEAK_L           0x000A
#define MAS_QPEAK_R           0x000B
#define MAS_DQPEAK_L          0x000C
#define MAS_DQPEAK_H          0x000D

#endif
