//*****************************************************************************
// File Name	: clock.c
// 
// Title		: Lab #3
// Revision		: 1.0
// Notes		:	
// Target MCU	: Atmel AVRmega323 
//*****************************************************************************

 
//----- Include Files ---------------------------------------------------------
#include <io.h>		// include I/O definitions (port names, pin names, etc)
#include <sig-avr.h>	// include "signal" names (interrupt names)
#include <interrupt.h>	// include interrupt support
#include <progmem.h>

#include "global.h"	// include our global settings
#include "rprintf.h"	// include printf function library
#include "timer.h"	// include timer function library
#include "uart.h"	// include timer function library

#include "./lcd.h"        // LCD driver
#include "./pcf8574.h"    // pcf8574 driver
#include "./ata.h"        // ata driver
#include "./fat.h"

//----- Defines ---------------------------------------------------------------

// Button definition
#define PROGRAM_BUTTON 4
#define UP_BUTTON 5
#define DOWN_BUTTON 6
#define ALARM_BUTTON 7

// Definition of button thresholds
#define PROGRAM_BUTTON_THRESHOLD 20 
#define ALARM_BUTTON_THRESHOLD 20 
#define UP_BUTTON_THRESHOLD 8 
#define DOWN_BUTTON_THRESHOLD 8 

// Different modes of the clock
#define DISPLAY_TIME 0
#define PROGRAM_CLOCK 1
#define PROGRAM_ALARM 2
#define PLAY_ALARM 3
#define CONTRAST_CONTROL 4

// Amount of time to blink.
#define BLINK_COUNT 16

// Custom characters used in LCD
#define PROGRAM_ICON 0
#define ALARM_ICON   1
#define WAVE_R1   2
#define WAVE_R2   3
#define WAVE_R3   4
#define WAVE_L1   5
#define WAVE_L2   6
#define WAVE_L3   7


//----- Functions -------------------------------------------------------------
void updateTime(void);
void printTime(unsigned short hours, unsigned short minutes, unsigned short seconds);
void program_clock(void);
void set_alarm(void);
void program_alarm(void);
void play_alarm(void);
void play_note_a(unsigned short index);
void play_note_b(unsigned short index);
void customChars(void);
void displayAlarmPlay(void);
void set_contrast(void);

//----- Global Variables ------------------------------------------------------
// current time
static unsigned short hours;
static unsigned short minutes;
static unsigned short seconds;

// alarm set time
static unsigned short alarm_hours;
static unsigned short alarm_minutes;
static unsigned short alarm_seconds;

// alarm on or off
static unsigned short alarm_status;

// other variables used throughout
static unsigned short lastseconds;
static unsigned short count;
static unsigned short program_count;
static unsigned short alarm_count;
static unsigned short up_count;
static unsigned short down_count;
static unsigned short temp_count;
static unsigned short time_modulo;
static unsigned short mode;
static unsigned short alarm_icon_state;
static unsigned short contrast;
static unsigned short buttons;
static unsigned short lastbuttons;


// Character map for the LCD screen
u08 __attribute__  ((progmem)) lcd_char_map[] = { 	
  0xFF, 0xF1, 0xF5, 0xF1, 0xF7, 0xF7, 0xF7, 0xFF,  
  0xE0, 0xE4, 0xEE, 0xEE, 0xEE, 0xFF, 0xE4, 0xE0,
  0xE0, 0xE0, 0xF0, 0xE8, 0xE8, 0xF0, 0xE0, 0xE0,
  0xE0, 0xE4, 0xE2, 0xE2, 0xE2, 0xE2, 0xE4, 0xE0,
  0xE4, 0xE2, 0xE1, 0xE1, 0xE1, 0xE1, 0xE2, 0xE4,
  0xE0, 0xE0, 0xE1, 0xE2, 0xE2, 0xE1, 0xE0, 0xE0,
  0xE0, 0xE4, 0xE8, 0xE8, 0xE8, 0xE8, 0xE4, 0xE0,
  0xE4, 0xE8, 0xF0, 0xF0, 0xF0, 0xF0, 0xE8, 0xE4 
};

static unsigned short silly1;
static unsigned short silly2;
static unsigned short silly3;


//----- Begin Code ------------------------------------------------------------
int main(void)
{
  unsigned long current_sector=2;
  u16 i;

  preCharge();
  
  // initialize the AVRlib libraries
  timerInit(); 			// initialize the timer system
  uartInit();                             // initialize the UART (serial port)
  uartSetBaudRate(57600);          // set the baud rate to 9600
  //  rprintfInit(uartSendByte); 
  //OK
  
  //rprintfInit(lcdDataWrite);// make all rprintf statements use lcd for output
  
  sbi(ASSR, AS2); // Use 32khz crystal
  timerAttach(TIMER2OVERFLOW_INT, updateTime);
  timer2SetPrescaler(0x02); // 16 interrupts per second

  // while(1) outb(PORTA, 0x00);
  timerPause(100);
  //while(1) outb(PORTA, 0x00);

  lcdInit();                    // initialize the LCD
  lcdSetContrast(100);

  i2cInit();
  
  rprintfInit(lcdDataWrite);
  rprintf("INIT");
  lcdReturn();
  timerPause(100);

  
  rprintfInit(uartSendByte);   
  ataInit();
  ataIdentify(buffer);
  //  ataPrintSector(buffer);
  
  fatInit();

  //  rprintf("GetDirEntry: %d\r\n", fatGetDirEntry(0));
  
  
  //readRegisters();
  
  // Good to know the LCD works
  //rprintf("INIT ");
  
  //for (i=0; i<512; i++) {
  //  buffer[i] = 0;
  //}
  
  
  //ataPrintSector(buffer);
  
  //lcdClear();
  
  //sei();
  
  //  while(1) {
  //silly = read8574();
  //lcdClear();
  //rprintf("%x", inb(TCNT2));
  //}
  
  // reset variable
  hours = 12;
  minutes = 0;
  seconds = 0;
  alarm_hours = 6;
  alarm_minutes = 0;
  alarm_seconds = 0;
  
  count = 0;
  program_count = 0;
  alarm_count = 0;
  alarm_status = 0;
  contrast = 0;
  mode = DISPLAY_TIME;
  
  sei(); // enable RTC!
  
  //ataPrintSector(buffer);

  //readRegisters();
  
  // enter endless while loop
  while (1) {
    
    if (lastseconds != seconds || buttons != lastbuttons) {  
      rprintfInit(lcdDataWrite);
      lastseconds = seconds;
      lastbuttons = buttons;
      
      lcdSetAddress(0x40);
      printTime(hours, minutes, seconds);
      rprintf(" %x", buttons);
      if (seconds % 10 == 5) {
	rprintfInit(uartSendByte);
	fatGetDirEntry(0);
	current_sector = 33;
	ataReadSector(current_sector, buffer);
	ataPrintSector(buffer);
	rprintf("GetDirEntry: %d\r\n", fatGetDirEntry()1);
	current_sector = fatGetDirEntry(1);
	current_sector = fatNextCluster(current_sector);
	//current_sector = fatNextCluster(current_sector);
      } 
    }
  }
}
or9
y-
TVor9Ny-


// The interrupt service routine, happens once every 1/16 of a second
// This ISR not only handles the update of the time, it also manages
// the different push button events.  Counters are used to figure out
// how long a button has been pressed.
void updateTime(void)
{
  if (++count == 16) {
    count = 0;	
    if (++seconds == 60) {
      seconds = 0;
      if (++minutes == 60) {
	minutes = 0;
	if (++hours == 24) {
	  hours = 0;
	}
      }
    }
  }
  buttons = read8574();
}



void preCharge() {
  u08 i;
  outb(PORTA, 0xFF);
  outb(PORTB, 0xFF);
  outb(PORTC, 0xFF);
  outb(PORTD, 0x7F);
  outb(DDRA, 0xFF);
  outb(DDRB, 0xFF);
  outb(DDRC, 0xFF);
  outb(DDRD, 0xFF);
  for (i=0; i<255; i++) {}
  outb(PORTA, 0x00);
  outb(PORTB, 0x00);
  outb(PORTC, 0x00);
  outb(PORTD, 0x00);
  for (i=0; i<255; i++) {}
  outb(PORTA, 0xFF);
  outb(PORTB, 0xFF);
  outb(PORTC, 0xFF);
  outb(PORTD, 0x7F);
  for (i=0; i<255; i++) {}
  
}

